import numpy as np
import matplotlib.pyplot as plt
from scipy import special
from scipy import interpolate
from matplotlib import rc,rcParams
from matplotlib.ticker import MultipleLocator,AutoMinorLocator
rc('text', usetex=True)
plt.style.use("MATPLOTLIB_RCPARAMS.sty") 

#LOADING FOREGROUND DATA
data_fg = np.genfromtxt('O3a_ranking_stats.txt',names=True,dtype=None)

blu_fg = data_fg['BOverlap']
rlu_fg = data_fg['Rgal']  # R^{galaxy}  for the SIS galaxy lens model, with a merger rate density given by Rmin in Eq. (A1).

lblu_fg = np.log10(blu_fg+1e-6)
lrlu_fg = np.log10(rlu_fg+1e-6)

e1 = data_fg['event1']
e2 = data_fg['event2']

idx = np.where(blu_fg>=50)[0]
print('Event1\t\tEvent2\t\tlog-BOverlap\t\tlog-R^gal\t\tCombined')
for ii in idx:
    print('{}\t{}\t{}\t\t{}\t\t{}\n'.format(e1[ii],e2[ii],lblu_fg[ii],lrlu_fg[ii],lblu_fg[ii]+lrlu_fg[ii]))

#LOADING SIMULATED DATA
data_bg = np.genfromtxt('background_ranking_stats.txt',names=True,dtype=None)

blu_bg = data_bg['BOverlap']
rlu_bg = data_bg['Rgal'] # R^{galaxy}  for the SIS galaxy lens model, with a merger rate density given by Rmin in Eq. (A1).

lblu_bg = np.log10(blu_bg+1e-6)
lrlu_bg = np.log10(rlu_bg+1e-6)

c_bg = blu_bg*rlu_bg # Combined Bayes factor
lc_bg = np.log10(c_bg+1e-6)

# Catalog FAP vs Threshold 
min(lc_bg), max(lc_bg)
b = np.linspace(-6.1, 7, 10000)
hh, bb = np.histogram(lc_bg,bins=b,density=True)
dc = bb[1]-b[0]
sim_BluRlu = 0.5*(bb[1:]+b[:-1])
sim_fap = 1-np.cumsum(hh)*dc

N_events = 38
N_pairs = 38*37/2.

# FAP for catalogue = 1-(1-FAP_{pair})^N; where N= number of pairs
cat_fap = 1-np.power(1-sim_fap, N_pairs) 
sim_BluRlu_of_fap = interpolate.interp1d(cat_fap, sim_BluRlu)

plt.figure(figsize=(4.4,4.))
x_vec = np.linspace(-3,5,1000)
ax = plt.subplot(111)

# calculate the Bayes factor values corresponding to different FAPs which
# are given in terms of Gaussian std deviations 
sigma = 2**-0.5

print('... Bayes factors log(Blu * Rlu) corresponding to different FAPs')
print('\t# sigma log(Blu * Rlu)')

for ii in [1,2]:
    BluRlu_sim = sim_BluRlu_of_fap(special.erfc(ii*sigma))
    print('\t', ii, '\t', BluRlu_sim)
    
    # plot contours of FAP and label them 
    plt.plot(x_vec, BluRlu_sim-x_vec, lw=1, ls='--', color='gray', alpha=0.5)
    
    idx = np.max(np.where((BluRlu_sim-x_vec) > +3)[0])
    plt.text(x_vec[idx]-0.7, BluRlu_sim-x_vec[idx]+0.8, '%d$\sigma$' %ii)

# plot Bayes factor from the events 
for ii in range(len(lblu_fg)):
    if lblu_fg[ii] > np.log10(50) and lrlu_fg[ii] > -2:  # Potting  event_pairs with Blu greater than 1
        ee1 = e1[ii][4:8].decode('utf-8')
        ee2 = e2[ii][4:8].decode('utf-8')
        print('{}\t{}\t{}\t{}'.format(ee1,ee2,lblu_fg[ii],lrlu_fg[ii]))

        plt.plot(lrlu_fg[ii], lblu_fg[ii], marker='.', ms=10, color='m', label='{}-{}'.format(ee1,ee2), ls='none')
        if(ee1=='0727' and ee2=='0731'):
            plt.text(lrlu_fg[ii]+0.1, lblu_fg[ii]-0.06, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0630' and ee2=='0513'):
            plt.text(lrlu_fg[ii]+0.07, lblu_fg[ii]+0.01, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0424' and ee2=='0503'):
            plt.text(lrlu_fg[ii]-0.1, lblu_fg[ii]+0.05, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0424' and ee2=='0521'):
            plt.text(lrlu_fg[ii]+0.04, lblu_fg[ii]-0.1, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0720' and ee2=='0930'):
            plt.text(lrlu_fg[ii]+0.05, lblu_fg[ii]-0.13, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0720' and ee2=='0728'):
            plt.text(lrlu_fg[ii]+0.05, lblu_fg[ii]-0.12, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0424' and ee2=='0803'):
            plt.text(lrlu_fg[ii]-0.04, lblu_fg[ii]-0.14, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0424' and ee2=='0413'):
            plt.text(lrlu_fg[ii]-0.2, lblu_fg[ii]+0.02, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0731' and ee2=='0910'):
            plt.text(lrlu_fg[ii]+0.05, lblu_fg[ii]+0.04, '{}-{}'.format(ee1,ee2),fontsize=10)
        elif(ee1=='0915' and ee2=='0719'):
            plt.text(lrlu_fg[ii]-0.55, lblu_fg[ii] + 0.02, '{}-{}'.format(ee1,ee2),fontsize=10)
        else:
            plt.text(lrlu_fg[ii]+0.1, lblu_fg[ii]+0.005, '{}-{}'.format(ee1,ee2),fontsize=10)


plt.xlim(-2.1,2.)
plt.ylim(1.5,5.)
plt.xlabel('$\log_{10}$ $\mathcal{R}^{\mathrm{gal}}$', fontsize=14)
plt.ylabel('$\log_{10}$ $\mathcal{B}^{\mathrm{overlap}}$', fontsize=14)

ax.tick_params(axis="x",which='major',length=3,bottom=True,left=True,direction='out')
ax.tick_params(axis="y",which='major',length=3,bottom=True,left=True,direction='out')

plt.tight_layout()
plt.grid(False)

plt.savefig('sec_5_Figure_3.pdf')
